@@ -25,6 +25,8 @@
 # 2017-09-02: added nice()
 # 2017-09-08: small changes to accommodate Effect.svyglm()
 # 2017-09-10: added replacement for ticksGrid()
+# 2018-05-09: fix typo in startup message
+# 2018-05-13: modified Analyze.model() to support partial-residual plots against factors.
 
 has.intercept <- function(model, ...) any(names(coefficients(model))=="(Intercept)")
 
@@ -46,7 +48,7 @@ mfrow <- function(n, max.plots=0){
 }
 
 expand.model.frame <- function (model, extras, envir = environment(formula(model)),
-                                na.expand = FALSE){  # modified version of R base function
+                        na.expand = FALSE){  # modified version of R base function
   f <- formula(model)
   data <- eval(model$call$data, envir)
   ff <- foo ~ bar + baz
@@ -278,14 +280,14 @@ Analyze.model <- function(focal.predictors, mod, xlevels, default.levels=NULL, f
   }
   if (partial.residuals){
     numeric.predictors <- sapply(focal.predictors, function(predictor) is.numeric.predictor(predictor, mod))
-    if (!any(numeric.predictors)) warning("there are no numeric focal predictors", "\n  partial residuals suppressed")
-    else{
-      x.var <- which(numeric.predictors)[1]
-      x.var.name <- focal.predictors[x.var]
-      if (is.null(mod$xlevels[[x.var.name]])){
-        x.var.range <- range(X[, focal.predictors[x.var]])
-        x[[x.var]][["levels"]] <- seq(from=x.var.range[1], to=x.var.range[2], length=100)
-      }
+    if (is.null(x.var)){
+      x.var <- if (any(numeric.predictors)) which(numeric.predictors)[1]
+      else 1
+    }
+    x.var.name <- focal.predictors[x.var]
+    if (is.numeric(X[, x.var.name]) && is.null(xlevels[[x.var.name]])){
+      x.var.range <- range(X[, focal.predictors[x.var]])
+      x[[x.var]][["levels"]] <- seq(from=x.var.range[1], to=x.var.range[2], length=100)
     }
   }
   x.excluded <- list()
@@ -495,7 +497,7 @@ effectsTheme <- function(strip.background=list(col=gray(seq(0.95, 0.5, length=3)
   else packageStartupMessage("Use the command",
                              "\n    lattice::trellis.par.set(effectsTheme())",
                              "\n  to customize lattice options for effects plots.",
-                             "\nSee ?efffectTheme for details.")
+                             "\nSee ?efffectsTheme for details.")
 }
 
 # to handle defaults for list-style arguments
